#ifndef Control_included
#define Control_included

#include "ControlData.h"
#include "EMData.h"
#include <qlist.h>
#include <qscrollview.h>
#include "Image.h"
#include "ImageMx.h"

class AutomaskD;
class DriftmaskD;

// rotational autocorrelation of various angles in an image
class RVEC {
public:
float v[4];

RVEC() { v[0]=v[1]=v[2]=v[3]=0; }

RVEC(EMData *d) { make(d); }

void make(EMData *d) {
	int i;
	float f=0;

	for (i=2; i<6; i++) {
		v[i-2]=rotdot(d,i);
		f+=v[i-2]*v[i-2];
	}

	f=sqrt(f);
	for (i=0; i<4; i++) v[i]/=f; 
		
}

float rotdot(EMData *d,int n) {
	int i;
	float r=0;
	EMData *e=d->copy();
	
	for (i=1; i<n; i++) {
		e->setRAlign(PI*2.0*i/(float)n);
		e->rotateAndTranslate();
		r+=e->dot(d);
	}
	r/=(n-1.0);
	delete e;
	return r;
}

float dot(RVEC &vec) {
	return vec.v[0]*v[0]+vec.v[1]*v[1]+vec.v[2]*v[2];
}

};


// a little class for maintaining an ordered list
class Olist {
public:
float *dot;
float *dx,*dy,*da;
int N;

Olist(int ntokeep) {
	N=ntokeep;
	dot=(float *)malloc(N*sizeof(float));
	dx=(float *)malloc(N*sizeof(float));
	dy=(float *)malloc(N*sizeof(float));
	da=(float *)malloc(N*sizeof(float));
	for (int i=0; i<N; i++) {
		dot[i]=-1.0e20;
		dx[i]=dy[i]=da[i]=0;
	}
}

~Olist() {
	free(dot);
	free(dx);
	free(dy);
	free(da);
}

void insert(float vdot, float vdx, float vdy, float vda) {
	int i,j;
	
	for (i=0; i<N; i++) if (dot[i]<vdot) break;

	if (i==N) { return; }

	for (j=N-1; j>i; j--) {
		dot[j]=dot[j-1];
		dx[j]=dx[j-1];
		dy[j]=dy[j-1];
		da[j]=da[j-1];
	}
	dot[i]=vdot;
	dx[i]=vdx;
	dy[i]=vdy;
	da[i]=vda;
}

};


class Control : public ControlData {
    Q_OBJECT

	enum { MEASURE, SELECT, DELETE, DRAW, HELIX };

	Image *imview;
	EMData *data;

	QList<EMData> ptcl;
	ImageMx *clipview;
	QString boxname;

	AutomaskD *amd;
	DriftmaskD *dmd;

	QList<EMData> abptcl;
	QList<RVEC> abvec;
	float absig,abmm;
	
	unsigned char *bestmap;
	
	int mode;

	float apix,scale;
	int boxsize,olap;
	
	int curshape;
	int mx0,my0,sx0,sy0;

protected:
	virtual void closeEvent ( QCloseEvent *e );	

public:

    Control(QWidget* parent = NULL,const char* name = NULL);

    virtual ~Control();

	void delClip(int n);
	int newClip(int x0,int y0,int x1, int y1,int n=-1,int showclip=0);
	void readimage(char *fsp,int num=-1,int denom=-1,int bsize=-1);
	void boxMoved(int n);
	EMData *ptclCorr(int x,int y,int whole=0);
	EMData *ptclCorrSVD(int x,int y,int whole=0);
	void autoBox2(float *thr);
	void autoBoxS(float *thr);
	void boxesFrom(EMData *map,int x0,int y0,int sx,int sy,float *thresh);
	void boxesFromSVD(EMData *map,int x0,int y0,int sx,int sy,float *thresh);
	void autoMask(int first,int last,float thr,float filt);
	int helixUpdate(int n);
	int helixClean(int n);
	void Mreadbdb(const char* file);


signals:

public slots:
	void setMeasure();
	void setHelix();
	void setSelect();
	void setDelete();
	void setDraw();
	void delAll();
	void autoBoxM();
	void autoBoxS();
	void autoBoxSVD();

	void setSize(const QString&);
	void setAPix(const QString&);
	void setOLap(const QString&);
	void setScale(const QString&);
	void setBC(int value);

	void imDown(QMouseEvent *);		// mousedown in big image view
	void imMove(QMouseEvent *);		// mousemove event
	void imUp(QMouseEvent *);
	void imResized(int,int);
	void imMoved(int,int);			// origin changed in image
	void panned(float,float);

	void redisplayBoxes(int n=-1);
	
	void autoBox1();

	void clipClick(QMouseEvent *e,int i,int x,int y);

	void driftMask(float thr,int preview);

	void adjustBoxes();
	void Mzerotonoise();
	void Mnormalize();
	void Mreadimage();
	void Msaveclips();
	void Msaveimage();
	void Msaveimage8();
	void Mcheckdrift();
	void Mreadbdb();
	void Mreadbdbimg();
	void Mfocalauto();
	void Msavebdb();
	void Mhelp();
	void Mabout();
	void Mquit();
	void Mnewboxsize();
	void Mgridbox();
	void Mautomask();
	void Munautomask();
	void Mstripefilter();
	void Mdriftmask();
	void Mcenalignint();
	void Mwienerfilter();
	void Msvdfilt();
	
	void Mmedian3();
	void Mmedian5();
	void Mmedian7();
	void Mmedian9();
	void doMedian(int v);
};
#endif // Control_included
