#include "EMIcos.h"
#include "icos.h"
#include "util.h"

#ifdef DEBUG
#include "assert.h"
#endif

/*
 * invalid constraints
 * 1. stamp = stamp1 = sizeof(title)
 * 2. for each row of image data,
 *    value of first int = value of last int = nx * sizeof(float)
 */

EMIcos::EMIcos(const char* filename) {
    header = new icosH();
    is_zipped = 0;
    right_order = true;
    valid_header = false;
    in = open_image_file(filename, &is_zipped);

    if (!in) {
	printf("Error: cannot open file '%s'\n", filename);
    }
    else {
	if (fread(header, sizeof(icosH), 1, in) == 1) {
	    right_order = check_byte_order((char*)&header->stamp);

	    if (!right_order) {
		swapint((char*)&header->stamp, 1);
		swapint((char*)&header->stamp1, 1);
		swapint((char*)&header->stamp2, 1);
	    }

	    int title_sz = sizeof(header->title);
	    if (header->stamp == header->stamp1 &&
		header->stamp == title_sz) {
		valid_header = true;
	    }

	    if (valid_header) {
		swapint((char*)&header->nx, 1);
		swapint((char*)&header->ny, 1);
		swapint((char*)&header->nz, 1);

#ifdef DEBUG
		assert(header->nx > 0);
		assert(header->ny > 0);
		assert(header->nz > 0);
#endif
	    }
	}
    }
}

EMIcos::~EMIcos() {
    delete header;
    header = 0;

    if (is_zipped) {
	pclose(in);
    }
    else {
	fclose(in);
    }
}

bool EMIcos::is_right_order() const {
    return right_order;
}

bool EMIcos::is_valid_header() const {
    return valid_header;
}

int EMIcos::get_nx() const {
    return header->nx;
}

int EMIcos::get_ny() const {
    return header->ny;
}

int EMIcos::get_nz() const {
    return header->nz;
}

const char* EMIcos::get_title() const {
    return header->title;
}




