#ifndef _cylinder_h_
#define _cylinder_h_

#include "PyList.h"
#include "Cylinder.h"
#include "PyVect.h"
#include <boost/smart_ptr.hpp>


class PyEMData;

class PyCylinder;
typedef boost::shared_ptr<PyCylinder> PyCylinder_ptr;

namespace python = boost::python;

class PyCylinder : public Cylinder {
public:
    PyCylinder();
    PyCylinder(const PyVect& center, const PyVect& axisdir,
	       float len = 5.4, float r = 2.5, float w = 0.9);
    
    PyVect py_Center();
    PyVect py_Axis();
    PyQuater py_QuaterForAxis();
    float py_PlaneAngleDiff(PyCylinder& next);
    float py_AxisAngleDiff(PyCylinder& next);
    float py_Angle2Center(PyCylinder& next);
    float py_NormDistance(PyCylinder& next); 
    float py_CenterDistance(PyCylinder& next); 
    bool py_if_merge(PyCylinder& next, float da_limit = 30. * PI / 180.,
		     float distance_limit = 3.2); 
    bool py_merge(PyCylinder& next);

    void py_setCenter(PyVect& cen);
    static const char* py_doc(const char* funcname);
};


python::ref Cylinder_getstate(const python::tuple& args,
			      const python::dictionary& keywords);
PyObject* Cylinder_setstate(const python::tuple& args,
			    const python::dictionary& keywords);


class PyCylinders : public Cylinders {
public:
    
    PyCylinders py_copy();
    PyCylinders py_subset(PyList data_list, int n);

    bool py_add(PyCylinder& cyl);    

    bool py_ReadListfile(const char * in = "-");
    bool py_WriteListfile(const char * out = "-");

    bool py_ReadInventor(const char * in = "-");
    bool py_WriteInventor(const char * out = "-");

    bool py_WriteVRML(const char *out = "-");
    bool py_ReadDejaVu(const char * in="-");

    bool py_ReadCosec(const char * in="-");
    
    PyCylinder_ptr get(int i);
    static const char* py_doc(const char* funcname);
};

python::ref Cylinders_getstate(const python::tuple& args,
			       const python::dictionary& keywords);
PyObject* Cylinders_setstate(const python::tuple& args,
			     const python::dictionary& keywords);



class PyObjectStat : public ObjectStat {
public:
    void py_SetObject(PyEMData& map, const PyList& obj);
    void py_SetObject(PyEMData& map);
    void py_EigenSortTo(PyObjectStat& obj);
    static const char* py_doc(const char* funcname);
};


python::ref ObjectStat_getstate(const python::tuple& args,
				const python::dictionary& keywords);
PyObject* ObjectStat_setstate(const python::tuple& args,
			      const python::dictionary& keywords);



#endif
