#ifndef _pyemdata_h_
#define _pyemdata_h_

#include "EMData.h"
#include "PyList.h"
#include "PyEuler.h"
#include <boost/smart_ptr.hpp>
#include <Python.h>

class PyEMData;

typedef boost::shared_ptr<PyEMData> PyEMData_ptr;
typedef boost::shared_ptr<imagicH> imagicH_ptr;
typedef boost::shared_ptr<mrcH> mrcH_ptr;
typedef boost::shared_ptr<SPIDERH> SPIDERH_ptr;
typedef boost::shared_ptr<PyEuler> Euler_ptr;


class PyEMData : public EMData {
private:
    static PyEMData null_emdata;
    static Float null_float;
    static XYData null_xydata;

    PyEMData* deref();
    float* deref(Float& f);
    XYData* deref(XYData& xy);

public:
    enum ImageType {
	IMAGE_ANY,
	IMAGE_SAL,
	IMAGE_PGM,
	IMAGE_ICOS,
	IMAGE_MRC,
	IMAGE_IMAGIC,
	IMAGE_IMAGIC3D,
	IMAGE_EMIM,
	IMAGE_SINGLE_SPIDER,
	IMAGE_SPIDER,
	IMAGE_GATAN,
	IMAGE_GATAN3,
	IMAGE_PIF,
	IMAGE_VTK,
	IMAGE_AMIRA
    };
    
public:
    PyEMData();
    ~PyEMData();

    PyObject* get_num_array();
    void set_num_array(PyObject* num_array, int array_sz);
    
    int readImage(const char* filename, ImageType image_type,
		  int n = 0, int nodata = 0);
    
    int writeImage(const char* filename, ImageType image_type,
		   int n = 0, int nolock = 0, int mode = MODE_float);
    
    PyEMData_ptr py_copy(int withfft = 0, int withparent =0);
    PyEMData_ptr py_copyHead();
    
    PyEMData_ptr py_clip(int x0, int y0, int w, int h);
    PyEMData_ptr py_clip(int x0, int y0, int z0, int w, int h, int d);
    void py_insertClip(PyEMData& block, int x0,  int y0);
    
    void py_calcHist(PyList hist_list, int n, float hmin = 0,
		     float hmax = 0, int add = 0);

    void py_setParent(PyEMData& data);
    PyEMData_ptr py_Parent();
    
    PyEuler py_getEuler(); 
    void py_setRAlign(PyEuler angle);

    imagicH_ptr py_getImagicHed();
    void py_setImagicHed(imagicH& hed);
    mrcH_ptr py_getMRCHed();
    SPIDERH_ptr py_getSpiHed();
    
    PyEMData_ptr py_doFFT();
    PyEMData_ptr py_doIFT();
    PyEMData_ptr py_doRadon();
    PyEMData_ptr py_vertACF(int maxdy);

    PyEMData_ptr py_convolute(PyEMData& with);
    PyEMData_ptr py_calcCCF(PyEMData& with, PyEMData& filter = null_emdata);
    PyEMData_ptr py_calcCCFX(PyEMData& with, int y0 = 0, int y1 = -1, int nosum = 0);
    PyEMData_ptr py_calcMCF(PyEMData& with, PyEMData& filter = null_emdata);

    PyEMData_ptr py_littleBigDot(PyEMData& with, int sigma = 0);
    
    float py_transAlign(PyEMData& with, int useparent, int intonly,
			int maxshift, Float& peak = null_float);
    float py_rotAlignPrecen(PyEMData& with);
    float py_rotAlign(PyEMData& with);
    
    float py_rotAlignCH(PyEMData& with, int irad = 8, int orad = 0);	
    float py_transAlign3d(PyEMData& with, int useparent = 0, int intonly = 0);	

    PyEMData_ptr py_RTFAlignRadon(PyEMData& with, int maxshift = -1,
				  PyEMData& thisf = null_emdata,
				  PyEMData& radonwith = null_emdata,
				  PyEMData& radonthis = null_emdata,
				  PyEMData& radonthisf = null_emdata);

    PyEMData_ptr py_RTAlignRadon(PyEMData& with, int maxshift = -1,
				 PyEMData& radonwith = null_emdata,
				 PyEMData& radonthis = null_emdata);

    PyEMData_ptr py_RTFAlignSlow(PyEMData& with,
				 PyEMData& flip = null_emdata,
				 int maxshift = -1);

    PyEMData_ptr py_RTFAlignSlowest(PyEMData& with,
				    PyEMData& flip = null_emdata,
				    int maxshift = -1);

    PyEMData_ptr py_RTFAlign(PyEMData& with = null_emdata,
			     PyEMData& flip = null_emdata,
			     int usedot = 1, int maxshift = -1);

    PyEMData_ptr py_RFAlign(PyEMData& with,
			    PyEMData& flip = null_emdata,
			    int imask = 0);
    
    PyEMData_ptr py_RTAlign(PyEMData& with, int usedot = 0, int maxshift = 0);

    void py_refineAlign(PyEMData& with, int mode = 1);
    PyEMData_ptr py_makeRFP(int premasked = 0, int unwrap = 1);
    PyEMData_ptr py_unwrap(int r1 = -1, int r2 = -1, int xs = -1,
			   int dx = 0, int dy = 0, int do360 = 0);
    
    void py_calcRCF(PyEMData& with, PyList sum_list, int NS);
    void py_applyRadFn(int n, float x0, float dx, PyList y_list, int interp = 1);
    void py_addRadNoise(int n, float x0, float dx, PyList y_list, int interp);	   
    void py_subtract(PyEMData& data);
    
    void py_normalizeTo(PyEMData& noisy, int keepzero = 0, int invert = 0,
			Float& mult = null_float,
			Float& add = null_float);	

    void py_leastSquareNormalizeTo(PyEMData& to, float low_thresh = FLT_MIN, 
				   float high_thresh = FLT_MAX,
				   Float& scale = null_float,
				   Float& shift = null_float);

    void py_maskNormalize(PyEMData& mask, int sigmatoo = 1); 
    void py_calcRadDist(int n, float x0, float dx, PyList y_list);
    void py_calcRadDist(int n, float x0, float dx, PyList y_list,
			float acen, float amwid);

    void py_calcAzDist(int n, float a0, float da, PyList data_list,
		       float rmin, float rmax);
    float py_dot(PyEMData& data, int evenonly = 0);
    float py_lcmp(PyEMData& noisy, int keepzero = 0, int invert = 0,
		  Float& scale = null_float,
		  Float& shift = null_float);	 

    float py_pcmp(PyEMData& data, XYData& snr = null_xydata);
    void py_makeAverageWT(PyList in, PyEMData& curves, XYData& sf);
    void py_makeAverageCTFCW(PyList in, XYData& sf, PyList SNR_list = PyList(0));    

    PyList py_ctfCurve(int type, XYData& sf = null_xydata);
    void py_ctfMap(int type, XYData& sf = null_xydata);
    
    float py_fscmp(PyEMData& with);
    PyList py_fsc(PyEMData& with);
    void py_makeAverage(PyList in, PyEMData& sigma = null_emdata);
    void py_makeAverageIter(PyList in);
    
    void py_makeAverageCTFC(PyList in, float filtr);
    void py_makeAverageCTFCWauto(PyList in, const char *outfile = 0);
    
    void py_makeMedian(PyList in);
    void py_add(PyEMData& in);
    void py_addIncoherent(PyEMData& in);
						
    void py_mult(PyEMData& in, int recip = 0);

    void py_commonLines(PyEMData& d1, PyEMData& d2, int mode = 0, 
			int steps = 180, int horiz = 0);
    
    void py_commonLinesR(PyEMData& data1, PyEMData& data2, 
			 int steps = 180, int horiz = 0);
				
    PyEMData_ptr py_project3d(float alt, float az,  float phi, 
			      int mode,  float thr = 1.0);
    PyEMData_ptr py_fftSlice(float alt, float az, float phi, int mode = 5);
    void py_interpFT3d(float x, float y, float z, PyList ret_list, int mode);

    float py_normSlice(PyEMData& slice, float alt, float az,
		       float phi, Float& phaseresid = null_float);

    void py_insertSlice(PyEMData& slice, float alt, float az,
			float phi, int mode = 5, float weight = 1.0);

    void py_insertSliceWF(PyEMData& slice, float alt, float az,
			  float phi, int mode, PyList SNR_list,
			  float padratio);
    void py_insertSlice3DReal(PyEMData& slice, float weight);

    PyList py_getCTF();
    void py_setCTF(PyList ctf);
    
    int py_CTFCmp(PyEMData& with);
    PyEMData_ptr py_iftSlice();
    PyEMData_ptr py_row(int n);
    PyEMData_ptr py_col(int n);
    void py_setRow(PyEMData& d, int n);
    void py_setCol(PyEMData& d, int n);
    PyEMData_ptr py_mxMult(PyEMData& second);
    float py_dist(PyEMData& second, int n = 0);

    static const char* py_doc(const char* funcname);
};

int fileCount(const char* filespec, String& filetypes);

void pylist2emlist(const PyList& pylist, List* emlist);
void emlist2pylist(List* emlist, PyList& pylist);

python::ref EMData_getstate(const python::tuple& args,
			    const python::dictionary& keywords);
PyObject* EMData_setstate(const python::tuple& args,
			  const python::dictionary& keywords);


#endif
