#include "PyEMHist.h"
#include "PyPickle.h"
#include <string.h>


PyEMHist::PyEMHist(const char* parse) :
    EMHist(const_cast<char*>(parse)) { }

const char* PyEMHist::py_getDir()
{
    return getDir();
}

enum EMHistState {
    EMHIST_ID,
    EMHIST_PPID,
    EMHIST_TYPE,
    EMHIST_FILE,
    EMHIST_INFO,
    EMHIST_COMMAND,
    EMHIST_TSTR,
    EMHIST_RTSTR,
    EMHIST_DIR,
    EMHIST_DICT,
    EMHIST_NUM_STATES
};

BOOST_PYTHON_BEGIN_CONVERSION_NAMESPACE
static python_extension_class_converters<PyEMHist> _PyEMHist_type;
BOOST_PYTHON_END_CONVERSION_NAMESPACE


static const char* emhist_get_str(char* str)
{
    if (!str) {
	return "";
    }

    return static_cast<const char*>(str);
}


python::ref EMHist_getstate(const python::tuple& args,
			    const python::dictionary& keywords)
{
   check_args(args, keywords, 1);

   PyEMHist& emhist = BOOST_PYTHON_CONVERSION::from_python(args[0].get(),
							   python::type<PyEMHist&>());
   python::tuple result(EMHIST_NUM_STATES);
   result.set_item(EMHIST_ID, emhist.id);
   result.set_item(EMHIST_PPID, emhist.ppid);
   result.set_item(EMHIST_TYPE, emhist.type);

   result.set_item(EMHIST_FILE, emhist_get_str(emhist.file));
   result.set_item(EMHIST_INFO, emhist_get_str(emhist.info));
   result.set_item(EMHIST_COMMAND, emhist_get_str(emhist.command));
   result.set_item(EMHIST_TSTR, emhist_get_str(emhist.tstr));
   result.set_item(EMHIST_RTSTR, emhist_get_str(emhist.rtstr));
   result.set_item(EMHIST_DIR, emhist_get_str(emhist.dir));
   
   python::ref mydict = python::getattr(args[0], "__dict__");
   result.set_item(EMHIST_DICT, mydict);

   return result.reference();
}

static void emhist_copy_str(char** str1, const char* str2)
{
    assert(str1 != 0);
    
    if (*str1) {
	delete [] *str1;
	*str1 = 0;
    }
    
    *str1 = new char[strlen(str2) + 1];
    strcpy(*str1, str2);
}
    

PyObject* EMHist_setstate(const python::tuple& args,
			  const python::dictionary& keywords)
{
    set_dict(args, keywords, EMHIST_NUM_STATES);

    PyEMHist& emhist = BOOST_PYTHON_CONVERSION::from_python(args[0].get(),
							    python::type<PyEMHist&>());

    python::tuple state = BOOST_PYTHON_CONVERSION::from_python(args[1].get(),
							       python::type<python::tuple>());
    emhist.id = get_long(state[EMHIST_ID]);
    emhist.ppid = get_int(state[EMHIST_PPID]);
    emhist.type = get_int(state[EMHIST_TYPE]);

    emhist_copy_str(&emhist.file, get_string(state[EMHIST_FILE]));
    emhist_copy_str(&emhist.info, get_string(state[EMHIST_INFO]));
    emhist_copy_str(&emhist.command, get_string(state[EMHIST_COMMAND]));
    emhist_copy_str(&emhist.tstr, get_string(state[EMHIST_TSTR]));
    emhist_copy_str(&emhist.rtstr, get_string(state[EMHIST_RTSTR]));
    emhist_copy_str(&emhist.dir, get_string(state[EMHIST_DIR]));
    
    return  python::detail::none();
}

