#ifndef _pyeuler_h_
#define _pyeuler_h_

#include "Euler.h"
#include "PyList.h"

namespace python = boost::python;

class PyEuler : public Euler {
public:
    enum { EULER_MATRIX_SIZE = 9 };
    
    PyEuler();
    PyEuler(float a1, float a2, float a3);
    PyEuler(float a1, float a2, float a3, int typ);
    PyEuler(float a1, float a2, float a3, float a0, int typ);
    PyEuler(PyList m);
    ~PyEuler();
    
    void py_setSym(const char *symmetry);
    void py_setAngle(PyList m);
    PyList py_mat();
    void py_print(const char *id = 0, FILE* out = stdout);
    void py_printMatrix(const char *id = 0, FILE* out = stdout);

    static const char* py_doc(const char* funcname);
    
    friend PyEuler operator*(PyEuler lhs, PyEuler rhs);
    friend PyEuler operator/(PyEuler lhs, PyEuler rhs) ;
    friend bool operator!=(PyEuler lhs, PyEuler rhs);
    friend bool operator==(PyEuler lhs, PyEuler rhs);

    PyEuler py_inverse();
    PyEuler py_EulerMult(PyEuler bc);
    PyEuler py_FirstSym();
    PyEuler py_NextSym();
    PyEuler py_SymN(int n);
    
};


python::ref Euler_getinitargs(PyEuler& euler);
python::ref Euler_getstate(const python::tuple& args,
			   const python::dictionary& keywords);
PyObject* Euler_setstate(const python::tuple& args,
			 const python::dictionary& keywords);
#endif





