#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <string.h>
#include "Axes2d.h"
#include "misc.h"

//inline int rnd(float v) { return (int)floor(v+.5); }
//inline int rnd(float v) { return (int)v; }

Axes2d::Axes2d( QWidget *parent, const char *name ): QWidget( parent, name ) {
//initMetaObject();                           // initialize meta object

qp=new QPainter();
tfont=new QFont("Times",18,QFont::Bold);
tmet=new QFontMetrics(*tfont);
lfont=new QFont("Times",14,QFont::Bold);
lmet=new QFontMetrics(*lfont);
sfont=new QFont("Times",12);
smet=new QFontMetrics(*sfont);

setLimits(fRect(0,0,1,1));
options=D2_OPT_FRAME;
xopt=D2_OPT_TIC|D2_OPT_SCALE;
yopt=D2_OPT_TIC|D2_OPT_SCALE;
ticspace.x=ticspace.y=.2;
minor[0]=minor[1]=0;
plots=new QList<Plot>;
sel[0].x=sel[1].x=0;
needupd=1;
qpm=new QPixmap(size());

actions.setAutoDelete(TRUE);

setAction(LeftButton,D2_ACT_XVALUE|D2_ACT_YVALUE|D2_ACT_MPOS);
setAction(MidButton,D2_ACT_INSPECT);
setAction(RightButton,D2_ACT_ZOOM);
setAction(LeftButton|ShiftButton,D2_ACT_YLOG);

textcolor=black;
axiscolor=black;
framecolor=black;
mousecolor=blue;

qpd=NULL;
inspect=NULL;

setBackgroundMode(NoBackground);

legx=width()*3/4;
legy=height()-alimit.y+20.0;

labfn=0;

fileToSave="";
}

Axes2d::~Axes2d() {
delete plots;
delete qp;
delete qpm;
delete tfont;
delete lfont;
if (inspect) delete inspect;
//delete sfont;
}

void Axes2d::resizeEvent ( QResizeEvent *qe ) {
//QWidget::resizeEvent(qe);
qpm->resize(size());
needupd=1;
setOptions(options,xopt,yopt);
}

void Axes2d::setOptions(int opt,int xo,int yo)  { 
options=opt|(options&(~D2_CLRINT)); 
xopt=xo;
yopt=yo;

aorigin.x=0;
aorigin.y=0;
asize.x=width();
asize.y=height()-1;
alimit.y=aorigin.y+asize.y;
alimit.x=aorigin.x+asize.x;

axislw=framelw=0;

if (options&D2_OPT_TITLE) alimit.y-=tmet->height()*1.1;
if (xopt&D2_OPT_SCALE) aorigin.y+=smet->height();
if (xopt&D2_OPT_LABEL) aorigin.y+=lmet->height();
if (yopt&D2_OPT_SCALE) aorigin.x+=smet->height();
if (yopt&D2_OPT_LABEL) aorigin.x+=lmet->height();

asize.x=alimit.x-aorigin.x;
asize.y=alimit.y-aorigin.y;
needupd=1;
//update();
}

int Axes2d::inside(float x,float y) {
if (x>=aorigin.x&&x<=alimit.x&&y>=aorigin.y&&y<=alimit.y) return 1;
return 0;
}

void Axes2d::savePNG(QString &fts) {
fileToSave=fts; 
needupd=1; 
update(); 
}

void Axes2d::print(QPrinter *dev) {
qpd=dev;
needupd=2;
paintEvent(NULL);
needupd=1;
qpd=NULL;
}

void Axes2d::paintEvent( QPaintEvent *pe ) {
char *tmp = NULL;
int i,j,i0,i1,z=0,lx,ly;
float xx,yy,w,ll,s;
char t[40];
Plot *p;
QWMatrix initmx;
QPoint p0,p1;

#ifdef DEBUG2
	printf("Axes2d %p: paint (%d %p)\n",this,needupd,qpd);
#endif

if (minor[0]<0) minor[0]=0;
if (minor[1]<0) minor[1]=0;

if (needupd||qpd) {
emit changed();

if ((options&D2_OPT_AUTOSCALE) && !(options&D2_OPT_ASDIS)) rescale();

qpm->fill();
if (qpd) {
	qp->begin(qpd);
	if (qpd->orientation()==QPrinter::Portrait) {
		qp->setViewport(54,(792-504*height()/width())/2,504,504*height()/width());
//		printf("%d %d %d %d\n",54,(792-504*height()/width())/2,504,504*height()/width());
	}
	else {
		qp->setViewport(72,(612-648*height()/width())/2,648,648*height()/width());
	}
	qp->setWindow(0,0,width(),height());
}
else qp->begin(qpm);

initmx=qp->worldMatrix();
qp->setPen(textcolor);

//printf("%f\t%f\t%f\t%f\n",dorigin.x,dorigin.y,dlimit.x,dlimit.y);

if (options&D2_OPT_TITLE) {
	qp->setFont(*tfont);
	qp->drawText(rnd((aorigin.x+alimit.x)/2.0-tmet->width(title)/2.0),
		tmet->ascent()+1,title);
}

if (xopt&D2_OPT_LABEL) {
	qp->setFont(*lfont);
	qp->drawText(rnd((aorigin.x+alimit.x)/2.0-lmet->width(xlabel)/2.0),
		height()-1-lmet->descent(),xlabel);
}

if (yopt&D2_OPT_LABEL) {
	qp->setFont(*lfont);
	qp->rotate(-90.0);
	qp->drawText(-rnd(height()-(aorigin.y+alimit.y)/2.0+lmet->width(ylabel)/2.0),
		lmet->ascent(),ylabel);
	qp->setWorldMatrix(initmx);
}

qp->setPen(QPen(framecolor,framelw));
if (options&D2_OPT_FRAME) qp->drawRect(rnd(aorigin.x),height()-rnd(aorigin.y),rnd(asize.x),-rnd(asize.y)-2);

if (options&D2_OPT_LR) {
	qp->drawLine(rnd(aorigin.x),height()-1-rnd(aorigin.y),rnd(alimit.x),height()-1-rnd(aorigin.y));
	qp->drawLine(rnd(aorigin.x),height()-1-rnd(aorigin.y),rnd(aorigin.x), height()-1-rnd(alimit.y));
}

qp->setPen(QPen(axiscolor,axislw));
if (xopt&D2_OPT_AXIS && ydtor(0)<height()-aorigin.y && ydtor(0)>height()-alimit.y) {
//	printf("%f %f %f\n",aorigin.y,ydtor(0),alimit.y);
	qp->drawLine(rnd(aorigin.x),rnd(ydtor(0)),rnd(alimit.x),rnd(ydtor(0)));
}

if (yopt&D2_OPT_AXIS && xdtor(0)>aorigin.x && xdtor(0)<alimit.x) {
	qp->drawLine(rnd(xdtor(0)),height()-1-rnd(aorigin.y),rnd(xdtor(0)),height()-1-rnd(alimit.y));
}

qp->setFont(*sfont);

if (xopt&D2_OPT_TIC || xopt&D2_OPT_GRID) {
	qp->setPen(QPen(axiscolor,axislw));
	if (ticspace.x<=0) ticspace.x=dsize.x/5.0;
	i0=(int)ceil(dorigin.x/ticspace.x*(minor[0]+1));
	i1=(int)floor(dlimit.x/ticspace.x*(minor[0]+1));
	if ((i1-i0)>400) i1=i0+5;
	
	if (xopt&D2_OPT_AXIS && ydtor(0)>aorigin.y && ydtor(0)<alimit.y) { yy=ydtor(0); z=1; }
	else { yy=aorigin.y; z=0; }
	for (i=i0; i<=i1; i++) {
		xx=xdtor(i*ticspace.x/(minor[0]+1));
		if (i%(minor[0]+1)==0) s=.4; else s=.2;
		if (options&D2_OPT_LR||options&D2_OPT_FRAME) {
			qp->drawLine(rnd(xx),height()-rnd(aorigin.y),rnd(xx),height()-rnd(aorigin.y+smet->height()*s+axislw));
		}
		if (options&D2_OPT_FRAME) {
			qp->drawLine(rnd(xx),height()-1-rnd(alimit.y),rnd(xx),height()-1-rnd(alimit.y-smet->height()*s-axislw));
		}
		if (xopt&D2_OPT_AXIS && z) {
			if (rnd(yy)<height()-rnd(aorigin.y) && rnd(yy)>height()-1-rnd(alimit.y)) 
				qp->drawLine(rnd(xx),rnd(yy),rnd(xx),rnd(yy+smet->height()*s/2.0+axislw));
		}
		if (xopt&D2_OPT_GRID && s>.3) {
			qp->drawLine(rnd(xx),height()-rnd(aorigin.y),rnd(xx),height()-1-rnd(alimit.y));
		}
		if (xopt&D2_OPT_GRIDM && s<.3) {
			qp->drawLine(rnd(xx),height()-rnd(aorigin.y),rnd(xx),height()-1-rnd(alimit.y));
		}
	}

	qp->setPen(textcolor);
	i0/=(minor[0]+1);
	i1/=(minor[0]+1);
	if ((options&D2_OPT_FRAME) || (options&D2_OPT_LR)) { yy=aorigin.y; z=0; }
	else yy=-ydtor(0)+height()-1;
	yy-=smet->ascent();
	if ((i1-i0)>400) i1=i0+5;
	if (xopt&D2_OPT_SCALE) {	
		if (!(xopt&D2_OPT_CSCA)) tmp=(char *)malloc(24);
		ll=-100000.0;
		for (i=i0; i<=i1; i++) {
			xx=xdtor(i*ticspace.x);
			if (xopt&D2_OPT_CSCA && labfn) tmp=labfn(i*ticspace.x,0);
			else sprintf(tmp,"%1.3g",i*ticspace.x);
			w=smet->width(tmp)/2.0;
			if (xx-w<aorigin.x-smet->height()/2.0 || xx+w>width() || xx-w<=ll || (i==0 && z)) continue; 	
			qp->drawText(rnd(xx-w),height()-1-rnd(yy),tmp);
			ll=xx+2.0*w;
		}
		if (!(xopt&D2_OPT_SCALE)) free(tmp);
	}
}

if (yopt&D2_OPT_TIC ) {
	if (yopt&D2_OPT_AXIS && xdtor(0)>aorigin.x && xdtor(0)<alimit.x) { xx=xdtor(0); z=1; }
	else { xx=aorigin.x; z=0; }

	if (!(yopt&D2_OPT_LOG)) {
		qp->setPen(QPen(axiscolor,axislw));
		if (ticspace.y<=0) ticspace.y=dsize.y/5.0;
		i0=(int)ceil(dorigin.y/ticspace.y*(minor[1]+1));
		i1=(int)floor(dlimit.y/ticspace.y*(minor[1]+1));
		if ((i1-i0)>100) i1=i0+5;
	
		for (i=i0; i<=i1; i++) {
			yy=ydtor(i*ticspace.y/(minor[1]+1));
			ytic(xx,yy,(i%(minor[1]+1))!=0);
		}

		if (minor[1]>=0) {
			i0/=(minor[1]+1);
			i1/=(minor[1]+1);
		}
		if ((options&D2_OPT_FRAME) || (options&D2_OPT_LR)) { xx=aorigin.x; z=0; }
		else xx=xdtor(0);
		xx-=smet->descent();			// ???
		
		qp->setPen(textcolor);
		if (yopt&D2_OPT_SCALE) {	
			ll=100000.0;
			if (!(yopt&D2_OPT_CSCA)) tmp=(char *)malloc(24);
			qp->rotate(-90.0);
			for (i=i0; i<=i1; i++) {
				yy=ydtor(i*ticspace.y);
				if (yopt&D2_OPT_CSCA && labfn) tmp=labfn(i*ticspace.y,0);
				else sprintf(tmp,"%1.3g",i*ticspace.y);
				w=smet->width(tmp)/2.0;
				if (yy-w<aorigin.y || yy+w>height() || yy-w>ll || (i==0 && z)) continue;
				qp->drawText(-rnd(yy+w-1),rnd(xx),tmp);
				ll=yy+2.0*w;
			}
			qp->setWorldMatrix(initmx);
			if (!(yopt&D2_OPT_CSCA)) free(tmp);
		}
	}
	else {		// log scale
		if (!(yopt&D2_OPT_CSCA)) tmp=(char *)malloc(24);
		
		qp->setPen(QPen(axiscolor,axislw));
		i1=(int)ceil(log10(yrtod(aorigin.y)));
		i0=(int)floor(log10(yrtod(alimit.y)));
		if (i0>i1 || i1-i0>200) {
			printf("ERROR: axis scaling messed up. i0=%d i1=%d aoy=%g aly=%g\n",i0,i1,aorigin.y,alimit.y);
			i0=i1=0;
		}
		for (i=i0; i<=i1; i++) {
			// major
			yy=ydtor(pow(10.0,(float)i));
			ytic(xx,yy,0);

			// label
			if (yopt&D2_OPT_SCALE) {	
				qp->setPen(textcolor);
				if (yopt&D2_OPT_CSCA && labfn) tmp=labfn(i*ticspace.y,0);
				else sprintf(tmp,"10^%d",i);
				w=smet->width(tmp)/2.0;
				if (height()-(yy+w-1)>aorigin.y && yy+w<(height())) {
					qp->rotate(-90.0);
					qp->drawText(-rnd(yy+w-1),rnd(xx),tmp);
					qp->setWorldMatrix(initmx);
				}
			}
			
			// minor			
			ll=pow(10.0,(float)i)*2;
			w=ll;
			for (j=1; j<5; j++) {
				yy=ydtor(ll);
				if (yy>height()-aorigin.y||yy<height()-alimit.y) continue;
				ytic(xx,yy,1);
				ll+=w;
			}
		}
		if (!(yopt&D2_OPT_CSCA)) free(tmp);
	}
}		

p0.setX(rnd(aorigin.x));
p0.setY(rnd(height()-1-aorigin.y));
p1.setX(p0.x()+rnd(asize.x));
p1.setY(p0.y()+rnd(-asize.y));
p0=qp->xForm(p0);
p1=qp->xForm(p1);

qp->setClipRect(p0.x(),p1.y(),p1.x()-p0.x(),p0.y()-p1.y());
qp->setClipping(TRUE);

// draw plots
qp->save();
qp->setWindow(0,0,width()*OVERSMP,height()*OVERSMP);
for (p=plots->first(); p!=NULL; p=plots->next()) {
	p->display(qp,this,rnd(aorigin.x*OVERSMP),rnd((height()-1-alimit.y)*OVERSMP),rnd(alimit.x*OVERSMP),rnd((height()-1-aorigin.y)*OVERSMP));
}
qp->restore();

// all legends drawn after plots
lx=legx;
ly=legy;
for (p=plots->first(); p!=NULL; p=plots->next()) {
	if (options&D2_OPT_LEGEND) ly+=p->legend(qp,this,lx,ly);
}
if (!qpd) qp->end();
needupd=0;
//setBackgroundPixmap(*qpm);
}

QPixmap qpm2(size());
//bitBlt( this, 0,0, qpm );
bitBlt( &qpm2, 0,0, qpm );

if (!qpd) {
	qp->begin(&qpm2);
//	qp->begin(this);
//	qp->eraseRect(0,0,width(),height());
}

qp->setPen(mousecolor);
// crosshairs
if (options&D2_OPT_XCH)
	qp->drawLine(rnd(sel[1].x),height()-1-rnd(aorigin.y+3),rnd(sel[1].x),height()-1-rnd(alimit.y-3));

if (options&D2_OPT_YCH)
	qp->drawLine(rnd(aorigin.x+3),rnd(sel[1].y),rnd(alimit.x-3),rnd(sel[1].y));

if ((options&D2_OPT_XRA)&&(options&D2_OPT_YRA))
	qp->drawRect(rnd(sel[0].x),rnd(sel[0].y),rnd((sel[1].x)-(sel[0].x)),rnd((sel[1].y)-(sel[0].y)));
else if (options&D2_OPT_XRA) {
	qp->drawLine(rnd(sel[0].x),height()-1-rnd(aorigin.y+3),rnd(sel[0].x),height()-1-rnd(alimit.y-3));
	qp->drawLine(rnd(sel[1].x),height()-1-rnd(aorigin.y+3),rnd(sel[1].x),height()-1-rnd(alimit.y-3));
}
else if (options&D2_OPT_YRA) {
	qp->drawLine(rnd(aorigin.x+3),rnd(sel[0].y),rnd(alimit.x-3),rnd(sel[0].y));
	qp->drawLine(rnd(aorigin.x+3),rnd(sel[1].y),rnd(alimit.x-3),rnd(sel[1].y));
}
	
if (options&D2_OPT_TXT) {
	if (xopt&D2_OPT_CSCA && labfn) sprintf(t,"x=%s (%1.3g)",labfn(xrtod(sel[1].x),0),xrtod(sel[1].x));
	else sprintf(t,"x=%1.3g",xrtod(sel[1].x));
	qp->drawText(rnd(alimit.x-12.0-smet->width(t)),height()-1-rnd(alimit.y-25.0),t);
	
	if (yopt&D2_OPT_CSCA && labfn) sprintf(t,"y=%s (%1.3g)",labfn(yrtod(sel[1].y),1),yrtod(sel[1].y));
	else sprintf(t,"y=%1.3g",yrtod(sel[1].y));
	qp->drawText(rnd(alimit.x-12.0-smet->width(t)),height()-1-rnd(alimit.y-45.0),t);
}

// select box
if (sel[0].x!=sel[1].x) {
}

qp->end();
bitBlt( this, 0,0, &qpm2 );
if (!fileToSave.isNull() && !fileToSave.isEmpty()) {
	qpm2.save(fileToSave,"PNG");
	fileToSave="";
}
//printf("axes paint end\n");
}

void Axes2d::ytic(float x,float y,float min) {
float s;

	if (min) s=.2; else s=.4;
	if (options&D2_OPT_LR||options&D2_OPT_FRAME) {
		qp->drawLine(rnd(aorigin.x),rnd(y),rnd(aorigin.x+smet->height()*s+axislw),rnd(y));
	}
	if (options&D2_OPT_FRAME) {
		qp->drawLine(rnd(alimit.x),rnd(y),rnd(alimit.x-smet->height()*s-axislw),rnd(y));
	}
	if (yopt&D2_OPT_AXIS && x>aorigin.x && x<alimit.x) {
		qp->drawLine(rnd(x),rnd(y),rnd(x+smet->height()*s/2.0+axislw),rnd(y));
	}
	if (yopt&D2_OPT_GRID && !min) {
		qp->drawLine(rnd(aorigin.x),rnd(y),rnd(alimit.x),rnd(y));
	}
	if (yopt&D2_OPT_GRIDM && min) {
		qp->drawLine(rnd(aorigin.x),rnd(y),rnd(alimit.x),rnd(y));	
	}
	
	return;
}

void Axes2d::setLimits(fRect r) {
float f;

if (options&D2_OPT_AUTOSCALE) options=(options&~D2_OPT_AUTOSCALE)|D2_OPT_ASDIS;

if (yopt&D2_OPT_LOG) {
	if (r.y0()<=0) r.sety0(-5.0); else r.sety0(log10(r.y0()));
	if (r.y1()<=0) r.sety0(-4.0); else r.sety1(log10(r.y1()));
}
dorigin.x=r.x0();
dorigin.y=r.y0();
dlimit.x=r.x1();
dlimit.y=r.y1();
dsize.x=r.xw();
dsize.y=r.yw();
sanity();
emit newLimits(r.x0(),r.y0(),r.x1(),r.y1());
plotChanged();
}

void Axes2d::setTitle(QString l) {
title=l;
options|=D2_OPT_TITLE;
needupd=1;
}

void Axes2d::setXLabel(QString l) {
xlabel=l;
xopt|=D2_OPT_LABEL;
needupd=1;
}

void Axes2d::setYLabel(QString l) {
ylabel=l;
yopt|=D2_OPT_LABEL;
needupd=1;
}

void Axes2d::setTFont(QFont &f) {
*tfont=f;
delete tmet;
tmet=new QFontMetrics(*tfont);
setOptions(options,xopt,yopt);
}

void Axes2d::setLFont(QFont &f) {
*lfont=f;
delete lmet;
lmet=new QFontMetrics(*lfont);
setOptions(options,xopt,yopt);
}

void Axes2d::setSFont(QFont &f) {
*sfont=f;
delete smet;
smet=new QFontMetrics(*sfont);
setOptions(options,xopt,yopt);
}

void Axes2d::setFonts(char *name,int titlesize,int labelsize,int scalesize) {
if (name) {
	tfont->setFamily(name);
	lfont->setFamily(name);
	sfont->setFamily(name);
}

if (titlesize>0||name) {
	tfont->setPointSize(titlesize);
	delete tmet;
	tmet=new QFontMetrics(*tfont);
}
if (labelsize>0||name) {
	lfont->setPointSize(labelsize);
	delete lmet;
	lmet=new QFontMetrics(*lfont);
}
if (scalesize>0||name) {
	sfont->setPointSize(titlesize);
	delete lmet;
	smet=new QFontMetrics(*sfont);
}
setOptions(options,xopt,yopt);
}

void Axes2d::addPlot(Plot *plot) { 
plots->append(plot); 
QObject::connect( plot, SIGNAL(changed()), this, SLOT(plotChanged()) );
needupd=1;
update();
}

void Axes2d::removePlot(Plot *plot) { 
plots->remove(plot); 
QObject::disconnect( plot, 0, this, 0 );
}

void Axes2d::noPlots() {
Plot *p;

for (p=plots->first(); p!=NULL; p=plots->next() )  QObject::disconnect( p, 0, this, 0 );
plots->clear(); 
}

void Axes2d::mousePressEvent(QMouseEvent *e) {
int x,y;
int *a,b;

x=e->x();
y=e->y();

sel[0].x=sel[1].x=x;
sel[0].y=sel[1].y=y;

emit mouseDown(e->state(),x,y);

//printf("p%d\n",e->state());
a=actions[e->state()|e->button()];
if (a) {
	b=*a;
	if (b&D2_ACT_XVALUE) options|=D2_OPT_XCH;
	if (b&D2_ACT_YVALUE) options|=D2_OPT_YCH;
	if (b&D2_ACT_XRANGE) options|=D2_OPT_XRA;
	if (b&D2_ACT_YRANGE) options|=D2_OPT_YRA;
	if (b&D2_ACT_MPOS) options|=D2_OPT_TXT;
	if (b&D2_ACT_ZOOM) options|=D2_OPT_XRA|D2_OPT_YRA;
	if (b&D2_ACT_YLOG) { yopt^=D2_OPT_LOG; rescale(); }
	if (b&D2_ACT_LEGEND) options|=D2_OPT_LEGEND;
}
update();
}

void Axes2d::mouseReleaseEvent(QMouseEvent *e) {
int x,y;
int *a,b;

x=e->x();
y=e->y();

sel[1].x=x;
sel[1].y=y;

emit mouseUp(e->state(),sel[0].x,sel[0].y,x,y);

//printf("r%d\n",e->state());
a=actions[e->state()];
if (a) {
	b=*a;
	if (b&D2_ACT_INSPECT) {
		if (!inspect) {
			inspect=new D2Inspector();
			inspect->setTarget(this);
		}
		inspect->show();
	}
	if (b&D2_ACT_ZOOM) {
		// rescale if no area was dragged
		if (fabs(sel[0].x-sel[1].x)<4 && fabs(sel[0].y-sel[1].y)<4) {
			options&=D2_CLRINT;
			rescale();
			return;
		}

		// don't allow too much zooming in x or y
		if (fabs(sel[0].x-sel[1].x)<4) { sel[0].x=aorigin.x; sel[1].x=alimit.x; }
		if (fabs(sel[0].y-sel[1].y)<4) { sel[0].y=aorigin.y; sel[1].y=alimit.y; }
	
		setLimits(fRect(xrtod(sel[0].x),yrtod(sel[0].y),xrtod(sel[1].x),yrtod(sel[1].y)));
		sel[0].x=sel[1].x=0;
		options&=D2_CLRINT;
		if (options&D2_OPT_AUTOSCALE) options|=D2_OPT_ASDIS;
		return;
	}
}

options&=D2_CLRINT;
}

void Axes2d::mouseMoveEvent(QMouseEvent *e) {
int x,y;
int *a,b;

x=e->x();
y=e->y();

sel[1].x=x;
sel[1].y=y;
//printf("d%d\n",e->state());

a=actions[e->state()|e->button()];
if (a) {
	b=*a;
	if (b&D2_ACT_LEGEND) { legx=x; legy=y; needupd=1; }
}

emit mouseDrag(e->state(),sel[0].x,sel[0].y,x,y);
update();
}

void Axes2d::mouseDoubleClickEvent(QMouseEvent *e) {
int *a,b;

a=actions[e->state()|e->button()];
if (a) {
	b=*a;
	if (b&D2_ACT_LEGEND) { options&=~D2_OPT_LEGEND; needupd=1; update(); }
}

}

void Axes2d::mouseEnter(int) { };
void Axes2d::mouseLeave(int) { };
void Axes2d::keyPressEvent(QKeyEvent *) { };
void Axes2d::keyReleaseEvent(QKeyEvent *) { };

void Axes2d::rescale() {
fRect cur;
Plot *p;

if (plots->count()<1) return;

if (options&D2_OPT_ASDIS) options=(options&~D2_OPT_ASDIS)|D2_OPT_AUTOSCALE;

cur=plots->first()->getLimits();
dorigin.x=cur.x0();
dorigin.y=cur.y0();
dlimit.x=cur.x1();
dlimit.y=cur.y1();

for (p=plots->first(); p!=NULL; p=plots->next()) {
	cur=p->getLimits();
	if (cur.x0()<dorigin.x) dorigin.x=cur.x0();
	if (cur.x1()>dlimit.x) dlimit.x=cur.x1();
	if (cur.y0()<dorigin.y) dorigin.y=cur.y0();
	if (cur.y1()>dlimit.y) dlimit.y=cur.y1();
}
if (yopt&D2_OPT_LOG) {
	if (dlimit.y<=0) dlimit.y=1.0;
	if (dorigin.y<=0) dorigin.y=dlimit.y/1.0e6;
	dorigin.y=log10(dorigin.y);
	dlimit.y=log10(dlimit.y);
}
else if (options&D2_OPT_SHOWY0){
	if (dorigin.y>0) dorigin.y=0;
	if (dlimit.y<0) dlimit.y=0;
}
dsize.x=dlimit.x-dorigin.x;
dsize.y=dlimit.y-dorigin.y;	

//printf("%f-%f  %f\n",dorigin.y,dlimit.y,dsize.y);

sanity();
emit newLimits(dorigin.x,dorigin.y,dlimit.x,dlimit.y);
plotChanged();
return;
}

void Axes2d::sanity() {
if (dsize.x<=0) { dlimit.x=dorigin.x+1.0; dsize.x=1.0; }
if (dsize.y<=0) { dlimit.y=dorigin.y+1.0; dsize.y=1.0; }
if (dsize.y>1.0e30) dsize.y=1.0e30;
if (dsize.x>1.0e30) dsize.x=1.0e30;
if (dsize.y<-1.0e30) dsize.y=-1.0e30;
if (dsize.x<-1.0e30) dsize.x=-1.0e30;
if (dorigin.x>1.0e30) dorigin.x=1.0e30;
if (dorigin.y>1.0e30) dorigin.y=1.0e30;
if (dorigin.x<-1.0e30) dorigin.x=-1.0e30;
if (dorigin.y<-1.0e30) dorigin.y=-1.0e30;


ticspace.x=pow(10.0,floor(log10(dsize.x)));
while (dsize.x/ticspace.x<4.0) ticspace.x/=2.0;

ticspace.y=pow(10.0,floor(log10(dsize.y)));
while (dsize.y/ticspace.y<4.0) ticspace.y/=2.0;

}

float Axes2d::ydtor(float y) { 
	float r;
	if (yopt&D2_OPT_LOG)
		r=(y>0?(log10(y)-dorigin.y):0)/dsize.y*asize.y+aorigin.y;
	else r=(y-dorigin.y)/dsize.y*asize.y+aorigin.y; 
//	if (r<aorigin.y) r=aorigin.y;
//	if (r>alimit.y) r=alimit.y;
	r=height()-r;
	return r;
}

float Axes2d::ydtor2(float y) { 
	float r;
	if (yopt&D2_OPT_LOG)
		r=(y>0?(log10(y)-dorigin.y):0)/dsize.y*asize.y+aorigin.y;
	else r=(y-dorigin.y)/dsize.y*asize.y+aorigin.y; 
//	if (r<aorigin.y) r=aorigin.y;
//	if (r>alimit.y) r=alimit.y;
	r=height()-r;
	return r*OVERSMP;
}

float Axes2d::yrtod(float y) { 
	y=height()-y;
	if (yopt&D2_OPT_LOG) return pow(10.0,(y-aorigin.y)/asize.y*dsize.y+dorigin.y);
	return (y-aorigin.y)/asize.y*dsize.y+dorigin.y; 
}

void Axes2d::plotChanged() { 

#ifdef DEBUG2
printf("Axes2d (%p): plot change\n",this);
#endif

if (!needupd) needupd=1;
update(); 
}

void Axes2d::setAction(int state,int act) { 
int *a=new int;

*a=act;
actions.replace(state,a);
}
