#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include "Column.h"
#include "DataSet.h"

//#define DEBUG11

Column::Column(int CID)  { 
//initMetaObject();                           // initialize meta object
data=NULL;
eqn=NULL;
parent=NULL; 
min=max=0;
n=1;
type=COL_RANGE; 
cid=CID;
}

Column::~Column()  { 
if (data) free(data); 
if (type==COL_EQN) { delete eqn; eqn=NULL; }
}

float Column::getMin() { 
	if (eqch) { updateEqn(); eqch=0; } 
	return min; 
}

float Column::getMax() { 
	if (eqch) { updateEqn(); eqch=0; } 
	return max; 
}

// Note that no 'changed' event is generated here. If necessary, the
// equation will generate one, and the Column will propegate it upwards
void Column::setVar(char *var,double val,bool quiet) { 
	if (type==COL_EQN) eqn->setVar(var,val,quiet);
}

void Column::setType(int t,int n) {

if (type==t) return;

if (t==COL_RANGE || t==COL_REPCYC) {
	if (data) { free(data); data=NULL; }
}
else {
	if (data) data=(float *)realloc(data,n*sizeof(float));
	else data=(float *)malloc(n*sizeof(float));
}

if (t==COL_EQN) {
	eqn=new Equation;
	QObject::connect(eqn,SIGNAL(changed()),this,SLOT(eqChanged()));
}
else {
	if (eqn) { delete eqn; eqn=NULL; }
}

if (t==COL_STRINGS) {

}
else {
	symbols.clear();
}

type=t;
}

void Column::setEqn(const char *neqn,int N,DataSet *par) {
setType(COL_EQN,N);
n=N;
parent=par;
//printf("Column eq %s -> %s\n",(const char *)ceqn,neqn);
min=max=0;
eqch=1;
eqn->parseEqn(neqn);

//emit changed();	// parse should send a change message here, which will get forwarded
}

void Column::setRepCyc(float Min,float Max,int nr,int nc,int N) {
setType(COL_REPCYC,N);
min=Min;
max=Max;
nrep=nr;
ncyc=nc;
n=N;
emit changed(cid);
}

void Column::setRange(float Min,float Max,int N) {
	setType(COL_RANGE,N);
	min=Min;
	max=Max;
	n=N;
	if (n<=0) n=1;
	
	emit changed(cid);
}

// passed array is 'given' to the column
void Column::setData(float *dat,int N) {
	int i;

	setType(COL_DATA,N);	
	if (N>=0) n=N;
	if (data && dat && data!=dat) free(data);
	if (type==COL_EQN) { delete eqn; eqn=NULL; }
	if (dat) data=dat;
	else {
		if (!data) {
			data=(float *)malloc(N*sizeof(float));
			memset(data,0,N*sizeof(float));
		}
	}
	type=COL_DATA;

	min=max=data[0];
	for (i=1; i<n; i++) {
		if (min>data[i]) min=data[i];
		if (max<data[i]) max=data[i];
	}
	if (min!=min) min=0;
	if (max!=max) max=min+.001;

	emit changed(cid);
}

void Column::setData(int n,float f) {

}

void Column::setData(int n,QString s) {

}

float Column::dataAt(int N) {
int i,j;

switch (type) {
case COL_RANGE:
	if (n==1) return min;
	return ((float)N/(float)(n-1))*(max-min)+min;
case COL_EQN:
	if (eqch) { updateEqn(); eqch=0; }
case COL_DATA:
	if (N<0||N>=n) return 0;
	return data[N];
case COL_REPCYC:
	j=n/(ncyc*nrep);
	if (j==1) return min;
	i=N%ncyc;
	i/=nrep;
	return ((float)i/(float)(j-1))*(max-min)+min;
}
return 0;
}

// This is called when an equation or variable changes
void Column::eqChanged() { 
#ifdef DEBUG2
printf("Column %p: eqn change\n",this);
#endif
if (type==COL_EQN) {
	eqch=1; 
	emit changed(cid); 
}
}


// Checks if this column depends on another within a DataSet
double *Column::dependsOn(int n) {
char s[5];
double *r;

if (type!=COL_EQN) return NULL;

sprintf(s,"x%1d",n);
r=eqn->getVarP(s);
if (r) return r;
return NULL;
}


void Column::updateEqn() {
int i,j;
double *row,**alt=NULL;
Column **src=NULL;

if (type!=COL_EQN) return;

#ifdef DEBUG2
printf("Column %p: Eqn update  ->\n");
#endif

row=eqn->getVarP("row");	// see if the equation uses the 'row' variable

// Other columns can be accessed as xn where n is the column number
// this initializes pointers for fast updates
if (parent) {
	alt=(double **)malloc(sizeof(double *)*parent->cols());
	src=(Column **)malloc(sizeof(Column *)*parent->cols());
	for (i=0; i<parent->cols(); i++) {
			// forces equation updates on any other accessed columns
		if ((alt[i]=dependsOn(i))) {
			parent->dataAt(i,0);
			src[i]=parent->getColumn(i);
//			connect(src[i],SIGNAL(changed()),this,SLOT(eqChanged()));
		}
//		else disconnect(parent->getColumn(i),0,this,0);
	}
}

min=1.0e30;
max=-1.0e30;

for (i=0; i<n; i++) {
	if (row) *row=(float)i;
	for (j=0; j<parent->cols(); j++) {
		if (src[j]&&alt[j]) alt[j][0]=src[j]->dataAt(i);
	}
	data[i]=eqn->eval();

	if (data[i]>max) max=data[i];
	if (data[i]<min) min=data[i];

}
if (min!=min) min=0;
if (max!=max) max=min+.001;

#ifdef DEBUG2
printf("Column %p: <- Eqn update\n");
#endif
if (alt) free(alt);
if (src) free(src);
}
